@@ -16,7 +16,7 @@
 // oneDPL headers should be included before standard headers
 #include <oneapi/dpl/random>
 
-#include <CL/sycl.hpp>
+#include <sycl/sycl.hpp>
 #include <iostream>
 #include <vector>
 
@@ -95,7 +95,7 @@ vector_example(sycl::queue& queue, std::uint32_t seed, std::vector<RealType>& x)
                         auto res = distr(engine);
 
                         // Store results from res to VecSize * offset position of x_acc
-                        res.store(idx.get_linear_id(), x_acc.get_pointer());
+                        res.store(idx.get_linear_id(), x_acc);
                     });
             });
     }
@@ -133,7 +133,7 @@ main()
         }
     };
 
-    sycl::queue queue(sycl::default_selector{}, async_handler);
+    sycl::queue queue(sycl::default_selector_v, async_handler);
 
     constexpr std::int64_t nsamples = 100;
     constexpr int vec_size = 4;
