@@ -402,13 +402,13 @@ __pattern_search_n(__parallel_tag<_IsVector>, _ExecutionPolicy&&, _RandomAccessI
 // copy_n
 //------------------------------------------------------------------------
 
-template <class _Tag, typename _ExecutionPolicy, typename = void>
+template <class _Tag, typename = void>
 struct __brick_copy_n;
 
-template <class _Tag, typename _ExecutionPolicy, typename = void>
+template <class _Tag, typename = void>
 struct __brick_copy;
 
-template <class _Tag, typename _ExecutionPolicy, typename = void>
+template <class _Tag, typename = void>
 struct __brick_move;
 
 //------------------------------------------------------------------------
@@ -836,7 +836,7 @@ __pattern_nth_element(__parallel_tag<_IsVector>, _ExecutionPolicy&&, _RandomAcce
 //------------------------------------------------------------------------
 // fill, fill_n
 //------------------------------------------------------------------------
-template <class _Tag, typename _ExecutionPolicy, typename _Tp, typename = void>
+template <class _Tag, typename _Tp, typename = void>
 struct __brick_fill;
 
 template <class _Tag, class _ExecutionPolicy, class _ForwardIterator, class _Tp>
@@ -847,7 +847,7 @@ template <class _IsVector, class _ExecutionPolicy, class _RandomAccessIterator,
 _RandomAccessIterator
 __pattern_fill(__parallel_tag<_IsVector>, _ExecutionPolicy&&, _RandomAccessIterator, _RandomAccessIterator, const _Tp&);
 
-template <class _Tag, typename _ExecutionPolicy, typename _Tp, typename = void>
+template <class _Tag, typename _Tp, typename = void>
 struct __brick_fill_n;
 
 template <class _Tag, class _ExecutionPolicy, class _OutputIterator, class _Size, class _Tp>
