@@ -18,12 +18,8 @@
 
 #include <iterator>
 #include <type_traits>
-#if _ONEDPL_CPP20_RANGES_PRESENT && _ONEDPL_CPP20_CONCEPTS_PRESENT
-#include <ranges> // std::ranges::contiguous_range
-#endif
 
 #include "../../utils_ranges.h"
-#include "../../ranges/nanorange.hpp" // contiguous_range from nanorange
 #include "../../iterator_impl.h"
 #include "sycl_iterator.h"
 #include "sycl_defs.h"
@@ -200,62 +196,6 @@ struct is_permutation<Iter, ::std::enable_if_t<Iter::is_permutation::value>> : :
 {
 };
 
-//is_passed_directly trait definition; specializations for the oneDPL iterators
-
-template <typename Iter, typename Void = void>
-struct is_passed_directly : ::std::is_pointer<Iter>
-{
-};
-
-//support legacy "is_passed_directly" trait
-template <typename Iter>
-struct is_passed_directly<Iter, ::std::enable_if_t<Iter::is_passed_directly::value>> : ::std::true_type
-{
-};
-
-//support std::vector::iterator with usm host / shared allocator as passed directly
-template <typename Iter>
-struct is_passed_directly<Iter, std::enable_if_t<oneapi::dpl::__internal::__is_known_usm_vector_iter_v<Iter>>>
-    : std::true_type
-{
-};
-
-template <typename Ip>
-struct is_passed_directly<oneapi::dpl::counting_iterator<Ip>> : ::std::true_type
-{
-};
-
-template <>
-struct is_passed_directly<oneapi::dpl::discard_iterator> : ::std::true_type
-{
-};
-
-template <typename Iter>
-struct is_passed_directly<::std::reverse_iterator<Iter>> : is_passed_directly<Iter>
-{
-};
-
-template <typename Iter, typename Unary>
-struct is_passed_directly<oneapi::dpl::transform_iterator<Iter, Unary>> : is_passed_directly<Iter>
-{
-};
-
-template <typename SourceIterator, typename IndexIterator>
-struct is_passed_directly<oneapi::dpl::permutation_iterator<SourceIterator, IndexIterator>>
-    : ::std::conjunction<
-          is_passed_directly<SourceIterator>,
-          is_passed_directly<typename oneapi::dpl::permutation_iterator<SourceIterator, IndexIterator>::IndexMap>>
-{
-};
-
-template <typename... Iters>
-struct is_passed_directly<zip_iterator<Iters...>> : ::std::conjunction<is_passed_directly<Iters>...>
-{
-};
-
-template <typename Iter>
-inline constexpr bool is_passed_directly_v = is_passed_directly<Iter>::value;
-
 // A trait for checking if iterator is heterogeneous or not
 
 template <typename Iter>
@@ -279,7 +219,17 @@ struct is_hetero_legacy_trait<Iter, ::std::enable_if_t<Iter::is_hetero::value>>
 };
 
 template <typename Iter>
-inline constexpr bool is_sycl_iterator_v = is_sycl_iterator<Iter>::value || is_hetero_legacy_trait<Iter>::value;
+using is_hetero_iterator = std::disjunction<is_sycl_iterator<Iter>, is_hetero_legacy_trait<Iter>>;
+
+template <typename Iter>
+inline constexpr bool is_hetero_iterator_v = is_hetero_iterator<Iter>::value;
+
+template <typename _Iter>
+using __is_passed_directly_device_ready =
+    std::conjunction<oneapi::dpl::is_indirectly_device_accessible<_Iter>, sycl::is_device_copyable<_Iter>>;
+
+template <typename _Iter>
+inline constexpr bool __is_passed_directly_device_ready_v = __is_passed_directly_device_ready<_Iter>::value;
 
 //A trait for checking if it needs to create a temporary SYCL buffer or not
 
@@ -289,8 +239,10 @@ struct is_temp_buff : ::std::false_type
 };
 
 template <typename _Iter>
-struct is_temp_buff<_Iter, ::std::enable_if_t<!is_sycl_iterator_v<_Iter> && !::std::is_pointer_v<_Iter> &&
-                                              !is_passed_directly_v<_Iter>>> : ::std::true_type
+struct is_temp_buff<_Iter,
+                    std::enable_if_t<!oneapi::dpl::__ranges::is_hetero_iterator_v<_Iter> && !std::is_pointer_v<_Iter> &&
+                                     !oneapi::dpl::__ranges::__is_passed_directly_device_ready_v<_Iter>>>
+    : std::true_type
 {
 };
 
@@ -523,7 +475,7 @@ struct __get_sycl_range
 
     //specialization for permutation_iterator using sycl_iterator as source
     template <sycl::access::mode _LocalAccMode, typename _It, typename _Map,
-              ::std::enable_if_t<is_sycl_iterator_v<_It>, int> = 0>
+              std::enable_if_t<oneapi::dpl::__ranges::is_hetero_iterator_v<_It>, int> = 0>
     auto
     __process_input_iter(oneapi::dpl::permutation_iterator<_It, _Map> __first,
                          oneapi::dpl::permutation_iterator<_It, _Map> __last)
@@ -531,7 +483,7 @@ struct __get_sycl_range
         auto __n = __last - __first;
         assert(__n > 0);
 
-        // Types for which oneapi::dpl::__ranges::is_sycl_iterator_v = true should have both:
+        // Types for which oneapi::dpl::__ranges::is_hetero_iterator_v = true should have both:
         //  "get_buffer()" to return the buffer they are base upon and
         //  "get_idx()" to return the buffer offset
 
@@ -551,7 +503,9 @@ struct __get_sycl_range
 
     //specialization for permutation_iterator using USM pointer or direct pass object as source
     template <sycl::access::mode _LocalAccMode, typename _Iter, typename _Map,
-              ::std::enable_if_t<!is_sycl_iterator_v<_Iter> && is_passed_directly_v<_Iter>, int> = 0>
+              std::enable_if_t<!oneapi::dpl::__ranges::is_hetero_iterator_v<_Iter> &&
+                                   oneapi::dpl::__ranges::__is_passed_directly_device_ready_v<_Iter>,
+                               int> = 0>
     auto
     __process_input_iter(oneapi::dpl::permutation_iterator<_Iter, _Map> __first,
                          oneapi::dpl::permutation_iterator<_Iter, _Map> __last)
@@ -567,9 +521,11 @@ struct __get_sycl_range
     }
 
     // specialization for general case, permutation_iterator with base iterator that is not sycl_iterator or
-    // passed directly.
+    // device accessible content iterators.
     template <sycl::access::mode _LocalAccMode, typename _Iter, typename _Map,
-              ::std::enable_if_t<!is_sycl_iterator_v<_Iter> && !is_passed_directly_v<_Iter>, int> = 0>
+              std::enable_if_t<!oneapi::dpl::__ranges::is_hetero_iterator_v<_Iter> &&
+                                   !oneapi::dpl::__ranges::__is_passed_directly_device_ready_v<_Iter>,
+                               int> = 0>
     auto
     __process_input_iter(oneapi::dpl::permutation_iterator<_Iter, _Map> __first,
                          oneapi::dpl::permutation_iterator<_Iter, _Map> __last)
@@ -578,7 +534,7 @@ struct __get_sycl_range
         assert(__n > 0);
 
         //TODO: investigate better method of handling this specifically for fancy_iterators which are composed fully
-        //      of a combination of fancy_iterators, sycl_iterators, and is_passed_directly types.
+        //      of a combination of fancy_iterators, sycl_iterators, and passed_directly types.
         //      Currently this relies on UB because the size of the accessor when handling sycl_iterators
         //      in recursion below this level is incorrect.
         auto res_src = this->operator()(__first.base(), __first.base() + 1 /*source size*/);
@@ -604,7 +560,8 @@ struct __get_sycl_range
 
     // for raw pointers and direct pass objects (for example, counting_iterator, iterator of USM-containers)
     template <sycl::access::mode _LocalAccMode, typename _Iter>
-    ::std::enable_if_t<is_passed_directly_v<_Iter>, __range_holder<oneapi::dpl::__ranges::guard_view<_Iter>>>
+    std::enable_if_t<oneapi::dpl::__ranges::__is_passed_directly_device_ready_v<_Iter>,
+                     __range_holder<oneapi::dpl::__ranges::guard_view<_Iter>>>
     __process_input_iter(_Iter __first, _Iter __last)
     {
         assert(__first < __last);
@@ -616,13 +573,13 @@ struct __get_sycl_range
     template <sycl::access::mode _LocalAccMode, typename _Iter>
     auto
     __process_input_iter(_Iter __first, _Iter __last)
-        -> ::std::enable_if_t<is_sycl_iterator_v<_Iter>,
-                              __range_holder<oneapi::dpl::__ranges::all_view<val_t<_Iter>, _LocalAccMode>>>
+        -> std::enable_if_t<oneapi::dpl::__ranges::is_hetero_iterator_v<_Iter>,
+                            __range_holder<oneapi::dpl::__ranges::all_view<val_t<_Iter>, _LocalAccMode>>>
     {
         assert(__first < __last);
         using value_type = val_t<_Iter>;
 
-        // Types for which oneapi::dpl::__ranges::is_sycl_iterator_v = true should have both:
+        // Types for which oneapi::dpl::__ranges::is_hetero_iterator_v = true should have both:
         //  "get_buffer()" to return the buffer they are base upon and
         //  "get_idx()" to return the buffer offset
 
@@ -701,7 +658,7 @@ struct __get_sycl_range
     //implementation of operator()(_Iter __first, _Iter __last) for the host iterator types
     template <sycl::access::mode _LocalAccMode, typename _Iter, typename _GetBufferFunc>
     auto
-    __process_host_iter_impl(_Iter __first, _Iter __last, _GetBufferFunc __get_buf)
+    __process_host_iter_impl([[maybe_unused]] _Iter __first, [[maybe_unused]] _Iter __last, _GetBufferFunc __get_buf)
     {
         static_assert(!oneapi::dpl::__internal::is_const_iterator<_Iter>::value ||
                           _LocalAccMode == sycl::access::mode::read,
@@ -758,55 +715,6 @@ __select_backend(const execution::fpga_policy<_Factor, _KernelName>&, _Ranges&&.
 }
 #endif
 
-// TODO: At some point with C++20, we should implement this with concepts to more easily support the less common edge
-// cases (e.g. a pipe over a counting iterator which is non-contiguous). For now, vectorization is primarily based on
-// range contiguity with specialization for internal views and guard views over internal iterators.
-// The following cases enable vectorization for a range:
-// 1. With C++20 concepts, the range satisfies std::ranges::contiguous_range.
-// 2. With C++17 nanoranges, the range satisfies __nanorange::nano::ranges::contiguous_range. Note that a view over
-// a SYCL buffer satisfies this concept along with pipe views that maintain access contiguity.
-// 3. The range is a guard view over an iterator with no global memory access: counting_iterator and discard_iterator
-// 4. The range is one of our internal, vectorizable range types: drop_view_simple, take_view_simple, or
-// transform_view_simple
-
-// Base case: check contiguous range properties
-template <typename _Rng>
-struct __is_vectorizable_range
-{
-    constexpr static bool value =
-#if _ONEDPL_CPP20_RANGES_PRESENT && _ONEDPL_CPP20_CONCEPTS_PRESENT
-        std::ranges::contiguous_range<_Rng> ||
-#endif
-        __nanorange::nano::ranges::contiguous_range<_Rng>;
-};
-
-// Basic guard view specializations - views which are not contiguous but do not interact with global memory
-template <typename _Ip>
-struct __is_vectorizable_range<oneapi::dpl::__ranges::guard_view<oneapi::dpl::counting_iterator<_Ip>>> : std::true_type
-{
-};
-
-template <>
-struct __is_vectorizable_range<oneapi::dpl::__ranges::guard_view<oneapi::dpl::discard_iterator>> : std::true_type
-{
-};
-
-// Recursive view specializations - internal views which we need to search inwards to identify if it is vectorizable
-template <typename _Rng, typename _F>
-struct __is_vectorizable_range<oneapi::dpl::__ranges::transform_view_simple<_Rng, _F>> : __is_vectorizable_range<_Rng>
-{
-};
-
-template <typename _Rng, typename _Size>
-struct __is_vectorizable_range<oneapi::dpl::__ranges::drop_view_simple<_Rng, _Size>> : __is_vectorizable_range<_Rng>
-{
-};
-
-template <typename _Rng, typename _F>
-struct __is_vectorizable_range<oneapi::dpl::__ranges::take_view_simple<_Rng, _F>> : __is_vectorizable_range<_Rng>
-{
-};
-
 } // namespace __ranges
 } // namespace dpl
 } // namespace oneapi
