@@ -66,7 +66,7 @@ struct run_copy_if
         ::std::fill_n(out_first, n, trash);
 
         // Run copy_if
-        auto i = copy_if(first, last, expected_first, pred);
+        [[maybe_unused]] auto i = copy_if(first, last, expected_first, pred);
         auto k = copy_if(exec, first, last, out_first, pred);
 #if !TEST_DPCPP_BACKEND_PRESENT
         EXPECT_EQ_N(expected_first, out_first, n, "wrong copy_if effect");
@@ -122,7 +122,7 @@ template <typename InputIterator, typename OutputIterator, typename OutputIterat
         ::std::fill_n(out_first, n, trash);
 
         // Run remove_copy_if
-        auto i = remove_copy_if(first, last, expected_first, [=](const T& x) { return !pred(x); });
+        [[maybe_unused]] auto i = remove_copy_if(first, last, expected_first, [=](const T& x) { return !pred(x); });
         auto k = remove_copy_if(exec, first, last, out_first, [=](const T& x) { return !pred(x); });
 #if !TEST_DPCPP_BACKEND_PRESENT
         EXPECT_EQ_N(expected_first, out_first, n, "wrong remove_copy_if effect");
@@ -209,7 +209,7 @@ struct test_non_const_remove_copy_if
             std::uint32_t i = (std::uint32_t)v;
             return i % 2 == 0;
         };
-        invoke_if(exec, [&]() { remove_copy_if(exec, input_iter, input_iter, out_iter, non_const(is_even)); });
+        remove_copy_if(exec, input_iter, input_iter, out_iter, non_const(is_even));
     }
 };
 
