@@ -28,14 +28,14 @@ main()
     constexpr bool just_call_submit = false;
     constexpr bool call_select_before_submit = true;
 
-    auto actual = test_initialization<policy_t, int>(u);
-    actual = test_select<policy_t, decltype(u), decltype(f)&, false>(u, f);
-    actual = test_submit_and_wait_on_event<just_call_submit, policy_t>(u, f);
-    actual = test_submit_and_wait_on_event<call_select_before_submit, policy_t>(u, f);
-    actual = test_submit_and_wait<just_call_submit, policy_t>(u, f);
-    actual = test_submit_and_wait<call_select_before_submit, policy_t>(u, f);
-    actual = test_submit_and_wait_on_group<just_call_submit, policy_t>(u, f);
-    actual = test_submit_and_wait_on_group<call_select_before_submit, policy_t>(u, f);
+    test_initialization<policy_t, int>(u);
+    test_select<policy_t, decltype(u), decltype(f)&, false>(u, f);
+    test_submit_and_wait_on_event<just_call_submit, policy_t>(u, f);
+    test_submit_and_wait_on_event<call_select_before_submit, policy_t>(u, f);
+    test_submit_and_wait<just_call_submit, policy_t>(u, f);
+    test_submit_and_wait<call_select_before_submit, policy_t>(u, f);
+    test_submit_and_wait_on_group<just_call_submit, policy_t>(u, f);
+    test_submit_and_wait_on_group<call_select_before_submit, policy_t>(u, f);
 
     return TestUtils::done();
 }
