@@ -98,7 +98,7 @@ test_body_for_loop_strided(Policy&& exec, Iterator first, Iterator last, Iterato
     using T = typename ::std::iterator_traits<Iterator>::value_type;
 
 #ifdef _PSTL_ICC_18_19_TEST_REVERSE_ITERATOR_WITH_STRIDE_BROKEN
-    if (isReverse<Iterator>::value)
+    if (is_reverse_v<Iterator>)
         return;
 #endif
 
@@ -108,7 +108,7 @@ test_body_for_loop_strided(Policy&& exec, Iterator first, Iterator last, Iterato
 
     ::std::experimental::for_loop_strided(exec, first, last, loop_stride, [&flip](Iterator iter) { flip(*iter); });
 
-    typename ::std::make_signed<Size>::type idx = 0;
+    ::std::make_signed_t<Size> idx = 0;
     for (auto iter = expected_first; iter != expected_last; ::std::advance(iter, single_stride), ++idx)
     {
         if (idx % loop_stride != 0)
@@ -186,7 +186,7 @@ test_body_for_loop_strided_n_integral(Policy&& exec, Iterator first, Iterator /*
     using T = typename ::std::iterator_traits<Iterator>::value_type;
 
 #ifdef _PSTL_ICC_18_19_TEST_REVERSE_ITERATOR_WITH_STRIDE_BROKEN
-    if (isReverse<Iterator>::value)
+    if (is_reverse_v<Iterator>)
         return;
 #endif
 
@@ -230,15 +230,14 @@ struct test_for_loop_impl
 };
 
 template <typename Policy, typename Iterator, typename Size, typename S>
-typename ::std::enable_if<
-    !::std::is_same<typename ::std::iterator_traits<Iterator>::iterator_category, ::std::forward_iterator_tag>::value,
-    void>::type
+::std::enable_if_t<
+    !::std::is_same_v<typename ::std::iterator_traits<Iterator>::iterator_category, ::std::forward_iterator_tag>>
 test_body_for_loop_strided_neg(Policy&& exec, Iterator first, Iterator /* last */, Iterator expected_first,
                                Iterator /* expected_last */, Size n, S loop_stride)
 {
     assert(loop_stride < 0);
 
-    using Ssize = typename ::std::make_signed<Size>::type;
+    using Ssize = ::std::make_signed_t<Size>;
 
     // Test negative stride value with non-forward iterators on range (first - 1, first)
     auto new_first = first;
@@ -256,9 +255,8 @@ test_body_for_loop_strided_neg(Policy&& exec, Iterator first, Iterator /* last *
 }
 
 template <typename Policy, typename Iterator, typename Size, typename S>
-typename ::std::enable_if<
-    ::std::is_same<typename ::std::iterator_traits<Iterator>::iterator_category, ::std::forward_iterator_tag>::value,
-    void>::type
+::std::enable_if_t<
+    ::std::is_same_v<typename ::std::iterator_traits<Iterator>::iterator_category, ::std::forward_iterator_tag>>
 test_body_for_loop_strided_neg(Policy&& /* exec */, Iterator /* first */, Iterator /* last */, Iterator /* expected_first */,
                                Iterator /* expected_last */, Size /* n */, S /* loop_stride */)
 {
@@ -295,11 +293,12 @@ test_for_loop()
 {
     for (size_t n = 0; n <= 10000; n = n <= 16 ? n + 1 : size_t(3.1415 * n))
     {
-        Sequence<T> inout(n, Gen<T>());
-        Sequence<T> expected = inout;
+        Sequence<T> in_out(n, Gen<T>());
+        Sequence<T> expected = in_out;
 
-        invoke_on_all_policies<>()(test_for_loop_impl(), inout.begin(), inout.end(), expected.begin(), expected.end(),
-                               inout.size());
+        // for_loop staff is implemented for the host policies only
+        invoke_on_all_host_policies()(test_for_loop_impl(), in_out.begin(), in_out.end(), expected.begin(),
+                                      expected.end(), in_out.size());
     }
 }
 
@@ -309,14 +308,15 @@ test_for_loop_strided()
 {
     for (size_t n = 0; n <= 10000; n = n <= 16 ? n + 1 : size_t(3.1415 * n))
     {
-        Sequence<T> inout(n, Gen<T>());
-        Sequence<T> expected = inout;
+        Sequence<T> in_out(n, Gen<T>());
+        Sequence<T> expected = in_out;
 
         ::std::vector<size_t> strides = {1, 2, 10, n > 1 ? n - 1 : 1, n > 0 ? n : 1, n + 1};
         for (size_t stride : strides)
         {
-            invoke_on_all_policies<>()(test_for_loop_strided_impl(), inout.begin(), inout.end(), expected.begin(),
-                                   expected.end(), inout.size(), stride);
+            // for_loop staff is implemented for the host policies only
+            invoke_on_all_host_policies()(test_for_loop_strided_impl(), in_out.begin(), in_out.end(), expected.begin(),
+                                          expected.end(), in_out.size(), stride);
         }
     }
 }
