@@ -130,6 +130,7 @@ DEFINE_TEST_2(test_inclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
             oneapi::dpl::inclusive_scan_by_segment(new_policy, keys_first, keys_last, vals_first, val_res_first);
         exec.queue().wait_and_throw();
 
+        EXPECT_TRUE(std::distance(val_res_first, res1) == n, "wrong return value, device policy");
         retrieve_data(host_keys, host_vals, host_res);
         check_values(host_keys.get(), host_vals.get(), host_res.get(), n);
 
@@ -142,6 +143,7 @@ DEFINE_TEST_2(test_inclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
                                                            val_res_first, BinaryPredicate());
         exec.queue().wait_and_throw();
 
+        EXPECT_TRUE(std::distance(val_res_first, res2) == n, "wrong return value, with predicate, device policy");
         retrieve_data(host_keys, host_vals, host_res);
         check_values(host_keys.get(), host_vals.get(), host_res.get(), n, BinaryPredicate());
 
@@ -154,6 +156,7 @@ DEFINE_TEST_2(test_inclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
                                                            val_res_first, BinaryPredicate(), BinaryOperation());
         exec.queue().wait_and_throw();
 
+        EXPECT_TRUE(std::distance(val_res_first, res3) == n, "wrong return value, with predicate and operator, device policy");
         retrieve_data(host_keys, host_vals, host_res);
         check_values(host_keys.get(), host_vals.get(), host_res.get(), n, BinaryPredicate(), BinaryOperation());
     }
@@ -169,23 +172,24 @@ DEFINE_TEST_2(test_inclusive_scan_by_segment, BinaryPredicate, BinaryOperation)
     operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Iterator2 vals_first, Iterator2 vals_last,
                Iterator3 val_res_first, Iterator3 val_res_last, Size n)
     {
-        typedef typename ::std::iterator_traits<Iterator1>::value_type KeyT;
-
         // call algorithm with no optional arguments
         initialize_data(keys_first, vals_first, val_res_first, n);
         auto res1 = oneapi::dpl::inclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first);
+        EXPECT_TRUE(std::distance(val_res_first, res1) == n, "wrong return value, no predicate, host policy");
         check_values(keys_first, vals_first, val_res_first, n);
 
         // call algorithm with predicate
         initialize_data(keys_first, vals_first, val_res_first, n);
         auto res2 = oneapi::dpl::inclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first,
                                                            BinaryPredicate());
+        EXPECT_TRUE(std::distance(val_res_first, res2) == n, "wrong return value, with predicate, host policy");
         check_values(keys_first, vals_first, val_res_first, n, BinaryPredicate());
 
         // call algorithm with predicate and operator
         initialize_data(keys_first, vals_first, val_res_first, n);
         auto res3 = oneapi::dpl::inclusive_scan_by_segment(exec, keys_first, keys_last, vals_first, val_res_first,
                                                            BinaryPredicate(), BinaryOperation());
+        EXPECT_TRUE(std::distance(val_res_first, res3) == n, "wrong return value, with predicate and operator, host policy");
         check_values(keys_first, vals_first, val_res_first, n, BinaryPredicate(), BinaryOperation());
     }
 
