@@ -13,11 +13,11 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "support/test_config.h"
+
 #include <oneapi/dpl/execution>
 #include <oneapi/dpl/algorithm>
 
-#include "support/test_config.h"
-
 #if _ENABLE_RANGES_TESTING
 #include <oneapi/dpl/ranges>
 #endif
@@ -26,7 +26,7 @@
 
 #include <iostream>
 
-int32_t
+std::int32_t
 main()
 {
 #if _ENABLE_RANGES_TESTING
@@ -37,7 +37,8 @@ main()
     sycl::buffer<int> A(max_n);
 
     auto iota = views::iota(0, max_n);
-    ::nano::ranges::copy(iota, views::host_all(A).begin());
+    //the name nano::ranges::copy is not injected into oneapi::dpl::experimental::ranges namespace
+    __nanorange::nano::ranges::copy(iota, views::host_all(A).begin());
     reverse(TestUtils::default_dpcpp_policy, A);
 
     for(auto v: views::host_all(A))
