@@ -13,17 +13,21 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include <oneapi/dpl/execution>
-
 #include "support/test_config.h"
 
+#include <oneapi/dpl/execution>
+
 #if _ENABLE_RANGES_TESTING
 #include <oneapi/dpl/ranges>
 #endif
 
 #include "support/utils.h"
 
+#include <cstddef>
 #include <iostream>
+#include <algorithm>
+#include <random>
+#include <vector>
 
 std::int32_t
 main()
@@ -35,13 +39,13 @@ main()
 
     using namespace TestUtils;
     using namespace oneapi::dpl::experimental::ranges;
+
+    auto exec = TestUtils::default_dpcpp_policy;
+    using Policy = decltype(TestUtils::default_dpcpp_policy);
     {
         sycl::buffer<int> A(data1, sycl::range<1>(max_n));
         sycl::buffer<int> B(data2, sycl::range<1>(max_n));
 
-        auto exec = TestUtils::default_dpcpp_policy;
-        using Policy = decltype(TestUtils::default_dpcpp_policy);
-
         sort(exec, A); //check passing sycl buffer directly
         sort(make_new_policy<new_kernel_name<Policy, 0>>(exec), all_view<int, sycl::access::mode::read_write>(B),
             ::std::greater<int>());
@@ -53,6 +57,29 @@ main()
 
     bool res2 = ::std::is_sorted(data2, data2 + max_n, ::std::greater<int>());
     EXPECT_TRUE(res2, "wrong effect from 'sort with comparator' with sycl ranges");
+
+    //test with random number and projection usage
+    ::std::default_random_engine gen{std::random_device{}()};
+    ::std::uniform_real_distribution<float> dist(0.0, 100.0);
+
+    constexpr ::std::size_t N = 1 << 20;
+    ::std::vector<int> keys(N);
+    ::std::generate(keys.begin(), keys.end(), [&] { return dist(gen); });
+    ::std::vector<int> values(keys);
+
+    {
+        sycl::buffer<int> A(values.begin(), values.end());
+        A.set_final_data(values.begin());
+        A.set_write_back(true);
+        sycl::buffer<int> B(keys.begin(), keys.end());
+        B.set_final_data(keys.begin());
+        B.set_write_back(true);
+
+        sort(make_new_policy<new_kernel_name<Policy, 1>>(exec), zip_view(views::all(A), views::all(B)), ::std::less{},
+             [](const auto& a) { return ::std::get<1>(a); });
+    }
+    bool res3 = ::std::is_sorted(values.begin(), values.end(), ::std::less{});
+    EXPECT_TRUE(res3, "wrong effect from 'sort by key'");
 #endif //_ENABLE_RANGES_TESTING
 
     return TestUtils::done(_ENABLE_RANGES_TESTING);
