@@ -22,10 +22,12 @@ main()
     using namespace test_std_ranges;
     namespace dpl_ranges = oneapi::dpl::ranges;
 
-    test_range_algo<0, int, data_in_in>{}(dpl_ranges::search,  std::ranges::search, binary_pred);
-    test_range_algo<1, int, data_in_in>{}(dpl_ranges::search,  std::ranges::search, binary_pred, proj, proj);
-    test_range_algo<2, P2, data_in_in>{}(dpl_ranges::search,  std::ranges::search, binary_pred, &P2::x, &P2::x);
-    test_range_algo<3, P2, data_in_in>{}(dpl_ranges::search,  std::ranges::search, binary_pred, &P2::proj, &P2::proj);
+    auto search_checker = TEST_PREPARE_CALLABLE(std::ranges::search);
+
+    test_range_algo<0, int, data_in_in>{big_sz}(dpl_ranges::search,  search_checker, binary_pred);
+    test_range_algo<1, int, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, proj, proj);
+    test_range_algo<2, P2, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, &P2::x, &P2::x);
+    test_range_algo<3, P2, data_in_in>{}(dpl_ranges::search,  search_checker, binary_pred, &P2::proj, &P2::proj);
 #endif //_ENABLE_STD_RANGES_TESTING
 
     return TestUtils::done(_ENABLE_STD_RANGES_TESTING);
