@@ -34,7 +34,7 @@
 // When such an issue is fixed, we must replace the usage of these "Latest" macros with the appropriate version number
 // before updating to the newest version in this section.
 
-#define _PSTL_TEST_LATEST_INTEL_LLVM_COMPILER 20250100
+#define _PSTL_TEST_LATEST_INTEL_LLVM_COMPILER 20250200
 
 #define _PSTL_TEST_LATEST_MSVC_STL_VERSION 143
 
@@ -78,13 +78,6 @@
 #define TEST_GCC10_EXCLUSIVE_SCAN_BROKEN (_GLIBCXX_RELEASE == 10)
 // GCC7 std::get doesn't return const rvalue reference from const rvalue reference of tuple
 #define _PSTL_TEST_GCC7_RVALUE_TUPLE_GET_BROKEN (_GLIBCXX_RELEASE > 0 && _GLIBCXX_RELEASE < 8)
-// Array swap broken on Windows because Microsoft implementation of std::swap function for std::array
-// call some internal function which is not declared as SYCL external and we have compile error
-#if defined(_MSC_VER)
-#   define TEST_XPU_ARRAY_SWAP_BROKEN (_MSC_VER <= 1937)
-#else
-#   define TEST_XPU_ARRAY_SWAP_BROKEN 0
-#endif
 
 #define _PSTL_SYCL_TEST_USM 1
 
@@ -282,6 +275,10 @@
     (_MSC_VER && TEST_DPCPP_BACKEND_PRESENT && __INTEL_LLVM_COMPILER < 20250100)
 
 // Intel(R) oneAPI DPC++/C++ compiler produces 'Unexpected kernel lambda size issue' error
-#define _PSTL_LAMBDA_PTR_TO_MEMBER_WINDOWS_BROKEN (_MSC_VER && TEST_DPCPP_BACKEND_PRESENT && __INTEL_LLVM_COMPILER < 20250200)
+#define _PSTL_LAMBDA_PTR_TO_MEMBER_WINDOWS_BROKEN (_MSC_VER && TEST_DPCPP_BACKEND_PRESENT && __INTEL_LLVM_COMPILER < 20250300)
+
+#if TEST_ONLY_HETERO_POLICIES && !TEST_DPCPP_BACKEND_PRESENT
+#    error "TEST_ONLY_HETERO_POLICIES is passed but device backend is not available"
+#endif
 
 #endif // _TEST_CONFIG_H
