@@ -14,10 +14,10 @@
 
 #include <oneapi/dpl/algorithm>
 
+#include "support/utils.h"
 #include "support/test_iterators.h"
 
 #include <cassert>
-#include <CL/sycl.hpp>
 
 struct Pred
 {
@@ -67,7 +67,7 @@ test(sycl::queue& deviceQueue)
 int
 main()
 {
-    sycl::queue deviceQueue;
+    sycl::queue deviceQueue = TestUtils::get_test_queue();
     test<input_iterator<const int*>, output_iterator<int*>>(deviceQueue);
     test<input_iterator<const int*>, input_iterator<int*>>(deviceQueue);
     test<input_iterator<const int*>, forward_iterator<int*>>(deviceQueue);
@@ -102,5 +102,6 @@ main()
     test<const int*, bidirectional_iterator<int*>>(deviceQueue);
     test<const int*, random_access_iterator<int*>>(deviceQueue);
     test<const int*, int*>(deviceQueue);
-    return 0;
+
+    return TestUtils::done();
 }
