@@ -14,10 +14,10 @@
 
 #include <oneapi/dpl/algorithm>
 
+#include "support/utils.h"
 #include "support/test_iterators.h"
 
 #include <cassert>
-#include <CL/sycl.hpp>
 
 template <class Iter>
 void
@@ -44,10 +44,11 @@ test(sycl::queue& deviceQueue)
 int
 main()
 {
-    sycl::queue deviceQueue;
+    sycl::queue deviceQueue = TestUtils::get_test_queue();
     test<forward_iterator<int*>>(deviceQueue);
     test<bidirectional_iterator<int*>>(deviceQueue);
     test<random_access_iterator<int*>>(deviceQueue);
     test<int*>(deviceQueue);
-    return 0;
+
+    return TestUtils::done();
 }
