@@ -13,16 +13,13 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include <iostream>
-
-#include <CL/sycl.hpp>
 #include <oneapi/dpl/functional>
 #include <oneapi/dpl/numeric>
 
-#include "support/test_iterators.h"
+#include <iostream>
 
-constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
-constexpr sycl::access::mode sycl_write = sycl::access::mode::write;
+#include "support/utils.h"
+#include "support/test_iterators.h"
 
 template <class T> class KernelTest;
 
@@ -33,7 +30,7 @@ template <typename _T1, typename _T2> void ASSERT_EQUAL(_T1 &&X, _T2 &&Y) {
 }
 
 template <class Iter> void test() {
-  sycl::queue deviceQueue;
+  sycl::queue deviceQueue = TestUtils::get_test_queue();
   int input[6] = {1, 2, 3, 4, 5, 6};
   int output[8] = {};
   sycl::range<1> numOfItems1{6};
@@ -43,8 +40,8 @@ template <class Iter> void test() {
     sycl::buffer<int, 1> buffer1(input, numOfItems1);
     sycl::buffer<int, 1> buffer2(output, numOfItems2);
     deviceQueue.submit([&](sycl::handler &cgh) {
-      auto in = buffer1.get_access<sycl_read>(cgh);
-      auto out = buffer2.get_access<sycl_write>(cgh);
+      auto in = buffer1.get_access<sycl::access::mode::read>(cgh);
+      auto out = buffer2.get_access<sycl::access::mode::write>(cgh);
       cgh.single_task<KernelTest<Iter>>([=]() {
         out[0] = oneapi::dpl::accumulate(Iter(&in[0]), Iter(&in[0]), 1,
                                          oneapi::dpl::multiplies<int>());
@@ -78,6 +75,5 @@ int main() {
   test<bidirectional_iterator<const int *>>();
   test<random_access_iterator<const int *>>();
   test<const int *>();
-  std::cout << "done" << std::endl;
-  return 0;
+  return TestUtils::done();
 }
